<?php
/**
 *    Executive loads the service class and executes the targeted method from the amfphp call
 * 
 *
 * @author    Musicman                        original design
 * @author    Justin Watkins                Gateway architecture, class structure, datatype io additions
 * @author    John Cowen                    Datatype io additions, class structure, 
 * @author    Klaasjan Tukker              Modifications, check routiines, and register-framework
 * @version    0.5
 */
class Executive
{
    var $_basecp = "services/";                                           // This is the directory "services/" relative to the gateway
    var $_classpath;                                                            // the classpath which is the path of the file from $_basecp
    var $_classname;                                                           // the string name of the class derived from the classpath
    var $_classConstruct;                                                    // the object we build from the class
    var $_methodname;                                                      // the method to execute in the construct
    var $_returnType;                                                         // the defined return type
    var $_instanceName;                                                    // the instance name to use for this gateway executive
    var $services = array();                                                // the list with registered service-classes
    var $_incomingcp;                                                        // The original incoming classpath
    var $_origClassPath;                                                    // The original classpath
    var $_headerFilter;                                                       // switch to take different actions based on the header
    
    /**
     * Executive constructor function
     */
    function Executive ()
    {
        // nothing really to do here yet?
    }
    
    /**
     * Sets the header current amfheader name. 
     * The executive will perform different actions depending on various
     * Headers passed by the remoting client
     *
     * @param header    A header string
     */
    function setHeaderFilter ($header)
    {
        $this->_headerFilter = $header;
    }
    
    /**
     *    Sets the class path so the Executive will know where to find service class files
     *
     * @param basecp    The base class path directory
     */
    function setBaseClassPath($basecp) {
        $this->_basecp = $basecp; 
    }
    
    /**
     *    setInstanceName binds this gateway to a string
     * When this happens only services with a matching instance name
     * may be used with this gateway instance
     *
     * @ param    name    The instance name to bind to
     */
    function setInstanceName ($name)
    {
        $this->_instanceName = $name;
    }
    
    /**
     * Through remoting you pass a . delimited path to the service class
     * without the extension.  This method translates that string into a real path
     *
     * @param cp    The full . delimited class path from the flash client
     */
    function setClassPath ($cp)
    {
        $this->_incomingcp = $cp;
        $lpos = strrpos($cp, ".");
        if ($lpos === false) {
            // throw an error because there has to be atleast 1
        } else {
            $this->_methodname = substr($cp, $lpos+1);
        }
        $trunced = substr($cp, 0, $lpos);
        $this->_origClassPath = $trunced;
        $lpos = strrpos($trunced, ".");
        if ($lpos === false) {
            $this->_classname = $trunced;
            $this->_classpath = $this->_basecp . $trunced.".php";
        } else {
            $this->_classname = substr($trunced, $lpos+1);
            $this->_classpath = $this->_basecp .str_replace(".", "/", $trunced) . ".php";
        }
    }

    /**
     *    Returns the return type for the called remote method
     * 
     * @returns        The defined return type from the service class's method table
     */
    function getReturnType ()
    {
        return $this->_returnType;
    }

    /**
     * The main method of the executive class. 
     * 
     * @param    a    Arguments to pass to the method
     */
    function doMethodCall ($a)
    {    
        $this->_calledMethod = $this->_methodname; 
        include_once ($this->_classpath);
        if(class_exists($this->_classname)) { // Just make sure the class name is the same as the file name
              $_finalclassname = "RemotingService_" . $this->_classname;  // append the class name to our RemotingService class
            if (!class_exists($_finalclassname)) { // only do this once
                $SuperClass = $this->_classname;
                include(AMFPHP_BASE."util/RemotingService.php");
                eval($RemotingSubClass); // evaluate the string as code
            }
        } else { // Class not found exception
            trigger_error("Class ".$this->_classname." does not exist", E_USER_ERROR);
        }
        $this->_classConstruct = new $_finalclassname($this->_classname); // build the extended service class
        $this->_classConstruct->setClassPath($this->_origClassPath); // pass the class path the the
    
        if ($this->_headerFilter == "DescribeService") { // catch the Describe service header
            $this->_methodname = "__describeService"; // override the method name if one was sent
            $this->_classConstruct->methodTable[$this->_methodname]['instance'] = $this->_instanceName;
        }
        
        if (isset($this->_classConstruct->methodTable[$this->_methodname])) { // check to see if the methodTable exists
            $methodrecord =& $this->_classConstruct->methodTable[$this->_methodname]; // create a shortcut for the ugly path
            if (isset($methodrecord['alias'])) { // see if it's an alias
                $this->_methodname = $methodrecord['alias'];
            }
            if (isset($this->_instanceName)) { // see if we have an instance defined
                if (!isset($methodrecord['instance']) || $this->_instanceName != $methodrecord['instance']) { // if the method doesn't or doesn't match die    
                    trigger_error($this->_headerFilter . " is not allowed to be used by the restricted gateway" ,E_USER_ERROR);
                }
            }
            if (isset($methodrecord['returns'])) { // check to see if it specifies a return type
                $this->_returnType = $methodrecord['returns'];
            } else {
                $this->_returnType = -1;
            }
            if ((isset($methodrecord['access'])) && (strtolower($methodrecord['access']) == "remote")) { // make sure we can remotely call it
                if (method_exists($this->_classConstruct, $this->_methodname)) { // finally see if it is actually defined
                    return call_user_func_array ( array(&$this->_classConstruct, $this->_methodname), $a); // do the magic
                } else { // method doesn't exist error
                    trigger_error("Function " . $this->_calledMethod . " does not exist in class ".$this->_classConstruct.".",E_USER_ERROR);
                }
            } else { // Access Denied error
                trigger_error("Access Denied to " . $this->calledMethod ,E_USER_ERROR);
            }
        } else { // method wasn't defined in the method table
            trigger_error("Function " . $this->_calledMethod . " does not exist in class ".$this->_classConstruct.".",E_USER_ERROR);
        }
    }
}
?>