<?php
// class used to turn the binary data into physical php objects.
class AMFDeserializer
{
	// the number of headers in the packet
	var $header_count;
	// the content of the headers
	var $headers;
	// the number of body elements
	var $body_count;
	// the content of the body
	var $body;
	// the object to store the deserialized data
	var $amfdata;
	
	
	/********************* PUBLIC METHODS ****************************/
	
	// constructor that also dserializes the raw data
	function AMFDeserializer (&$is)
	{
		$this->amfdata = new AMFObject();
		// save the input stream in this object
		$this->inputStream = &$is;
		// read the binary header
		$this->readHeader();
		// read the binary body
		$this->readBody();
		
	}
	
	// returns the instance of the AMFObject class
	function getAMFObject ()
	{
		return $this->amfdata;
	}
	
	/********************* PRIVATE METHODS ****************************/
	
	function readHeader ()
	{
		// ignore the first two bytes -- version or something
		$this->inputStream->readInt();
		// find the total number of header elements
		$this->header_count = $this->inputStream->readInt();
		// loop over all of the header elements
		while($this->header_count--)
		{
			$name = $this->inputStream->readUTF();
			// find the must understand flag
			$required = $this->readBoolean();
			// grab the length of the header element
			$length = $this->inputStream->readLong();
			// grab the type of the element
			$type = $this->inputStream->readByte();
			// turn the element into real data
			$content = $this->readData($type);
			// save the name/value into the headers array
			$this->amfdata->addHeader($name, $required, $content);
		}
	}
	
	function readBody ()
	{
		// find the total number of body elements
		$this->body_count = $this->inputStream->readInt();
		// loop over all of the body elements
		while($this->body_count--)
		{	
			$target = $this->readString();
			// the response that the client understands
			$response = $this->readString();
			// grab the length of the body element
			$length = $this->inputStream->readLong();
			
			// grab the type of the element
			$type = $this->inputStream->readByte();
			// turn the argument elements into real data
			$data = $this->readData($type);
			// add the body element to the body object
			$this->amfdata->addBody($target, $response, $data);
		}
	}
	
	
	// reads an object and converts the binary data into a PHP array structure
	function readObject ()
	{
		// init the array
		$ret = array();
		
		// grab the key
		$key = $this->inputStream->readUTF();
		 
		for  ($type = $this->inputStream->readByte(); $type != 9; $type = $this->inputStream->readByte())
		{	
			// grab the value
			$val = $this->readData($type);
			// save the name/value pair in the array
			$ret[$key] = $val;
			// get the next name
			$key = $this->inputStream->readUTF();
		}
		// return the array
		return $ret;
	}
	
	// reads and array object and converts the binary data into a PHP array
	function readArray ()
	{
		// init the array object
		$ret = array();
		// get the length of the array
		$length = $this->inputStream->readLong();
		// loop over all of the elements in the data
		for ($i=0; $i<$length; $i++)
		{
			// grab the type for each element
			$type = $this->inputStream->readByte();
			// grab each element
			$ret[] = $this->readData($type);
		}
		// return the data
		return $ret;
		
	}
	
	function readCustomClass ()
	{
		// grab the explicit type -- I'm not really convinced on this one but it works,
		// the only example i've seen is the NetDebugConfig object
		$typeIdentifier = $this->inputStream->readUTF();
		// the rest of the bytes are an object without the 0x03 header
		$value = $this->readObject();
		// save that type because we may need it if we can find a way to add debugging features
		$value["_explicitType"] = $typeIdentifier;
		// return the object
		return $value;
		
	}
	
	function readNumber ()
	{
		// grab the binary representation of the number
		return $this->inputStream->readDouble();	
	}
	
	// read the next byte and return it's boolean value
	function readBoolean ()
	{
		// grab the int value of the next byte
		$int = $this->inputStream->readByte();
		// if it's a 0x01 return true else return false
		if ($int == 1) return true;
		else return false;
	}
	
	function readString ()
	{
		return $this->inputStream->readUTF();
	}

	function readDate () // date reading function
	{
		$ms = $this->inputStream->readDouble(); // date in milliseconds from 01/01/1970

		// nasty way to get timezone 
		$int = $this->inputStream->readInt();
		if($int > 720){
			$int = -(65536 - $int);
			}
		$hr = floor($int / 60);
		$min = $int % 60;
		$timezone = "GMT " . -$hr . ":" . abs($min);
		// end nastiness 

		// is there a nice way to return entire date(milliseconds and timezone) in PHP???
		return $ms; 
	}

	// XML comes in as a plain string except it has a long displaying the length instead of a short?
	function readXML () //XML reading function
	{
		 // reads XML
		$rawXML = $this->inputStream->readLongUTF();
		
		// maybe parse the XML into a PHP XML structure??? or leave it to the developer
		
		// return the xml
		return $rawXML;
	}
	function readFlushedSO ()
	{
		// receives [type(07) 00 00] if SO is flushed and contains 'public' properties
		// see debugger readout ???
		return $this->inputStream->readInt();
	}

	function readASObject ()
	{
		/*
		object Button, object Textformat, object Sound, object Number, object Boolean, object String, 
		SharedObject unflushed, XMLNode, used XMLSocket??, NetConnection,
		SharedObject.data, SharedObject containing 'private' properties

		the final byte seems to be the dataType -> 0D
		*/
		return null;
	}
	
	// main switch function to process all of the data types
	function readData($type)
	{	
		switch($type)
		{	
			case 0: // number
				$data = $this->readNumber();
				break;
			case 1: // boolean
				$data = $this->readBoolean();
				break;
			case 2: // string
				$data = $this->readString();
				break;
			case 3: // object Object
				$data = $this->readObject();
				break;
			case 5: // null
				$data = null;
				break;
			case 6: // undefined
				$data = null;
				break;
			case 7: // flushed SharedObject containing 'public' properties
				$data = $this->readFlushedSO(); 
				break;
			case 8: // array
				// shared object format only (*.sol) don't think this exists in amf but who knows???
				break;
			case 10: // array
				$data = $this->readArray();
				break;
			case 11: // date
				$data = $this->readDate();
				break;
			case 13: // mainly internal AS objects
				$data = $this->readASObject();
				break;
			case 15: // XML
				$data = $this->readXML();
				break;
			case 16: // Custom Class
				$data = $this->readCustomClass();
				break;
			default: // unknown case
				print "xxx $type ";
				break;
		}
			return $data;
	}
	
	
}
?>
