<?php

class AMFSerializer
{
	// holder for the data
	var $data;
	// holder for the output stream
	var $out;
	
	function AMFSerializer(&$stream)
	{	
		// save
		$this->out = &$stream;
	}
	
	function serialize($d)
	{
		$this->amfout = $d;
		// write the version ???
		$this->out->writeInt(0);
		
		// get the header count
		$count = $this->amfout->numHeader();
		// write header count
		$this->out->writeInt($count);
		
		for ($i=0; $i<$count; $i++)
		{
			$this->writeHeader($i);
		}
		
		
		$count = $this->amfout->numBody();
		// write the body count
		$this->out->writeInt($count);
		
		for ($i=0; $i<$count; $i++)
		{
			// start writing the body
			$this->writeBody($i);
		}
	}
	
	function writeHeader ($i)
	{

		
		// for all header values
		// write the header to the output stream
		// ignoring header for now
	}

	function writeBody ($i)
	{
		$body = $this->amfout->getBodyAt($i);
		// write the responseURI header
		$this->out->writeUTF($body["target"]);
		// write null, haven't found another use for this
		$this->out->writeUTF($body["response"]);
		// always, always there is four bytes of FF, which is -1 of course
		$this->out->writeLong(-1);
		// write the data to the output stream
		$this->writeData($body["value"], $body["type"]);

	}
	
	// writes a boolean
	function writeBoolean ($d)
	{
		// write the boolean flag
		$this->out->writeByte(1);
		// write the boolean byte
		$this->out->writeByte($d);
	}
	// writes a string under 65536 chars, a longUTF is used and isn't complete yet
	function writeString ($d)
	{
		// write the string code
		$this->out->writeByte(2);
		// write the string value
//		$this->out->writeUTF(utf8_encode($d));
		$this->out->writeUTF($d);
	}
	
	function writeXML ($d)
	{
		$this->out->writeByte(15);
//		$this->out->writeLongUTF(utf8_encode($d));
		$this->out->writeLongUTF($d);		
	}

	// must be used PHPRemoting with the service to set the return type to date
	// still needs a more in depth look at the timezone
	function writeDate ($d)
	{
		// write date code
		$this->out->writeByte(11);
		// write date (milliseconds from 1970)
		$this->out->writeDouble($d);
		// write timezone
		// ?? this is wierd -- put what you like and it pumps it back into flash at the current GMT ?? 
		// have a look at the amf it creates...
		$this->out->writeInt(0); 
	}
	
	// write a number formatted as a double with the bytes reversed
	// this may not work on a Win machine because i believe doubles are
	// already reversed, to fix this comment out the reversing part
	// of the writeDouble method
	function writeNumber ($d)
	{
		// write the number code
		$this->out->writeByte(0);
		// write the number as a double
		$this->out->writeDouble($d);
	}
	// write null
	function writeNull ()
	{
		// null is only a 0x05 flag
		$this->out->writeByte(5);
	}

	// write array
	// since everything in php is an array this includes arrays with numeric and string indexes
	function writeArray ($d)
	{
		// make damn certain the input is an array
		if (is_array($d))
		{
			// assume we have a numeric array
			$num_array = 1;
			// grab the total number of elements
			$len = count($d);
			// loop over each element
			foreach($d as $key => $data)
			{	
				// make sure the keys are numeric
				if(!is_int($key) || ($key < 0))
				{
					// if they are not then this is an object	
					$num_array = 0;
					break;
				}
			}
			// all of the keys were numbers
			if($num_array)
			{	
				// write the numeric array code
				$this->out->writeByte(10);
				// write the count of items in the array
				$this->out->writeLong($len);
				// write all of the array elements
				for($i=0 ; $i < $len ; $i++)
					$this->writeData($d[$i]);
			}
			else
			{
				// this is an object so write the object code
				$this->out->writeByte(3);
				// write the object name/value pairs	
				$this->writeObject($d);
			}
		}
	}
	// writes an object to the stream
	function writeObject ($d)
	{
		// loop over each element
		foreach($d as $key => $data)
		{	
			// write the name of the object
			$this->out->writeUTF($key);
			// write the value of the object
			$this->writeData($data);
		}
		// write the end object flag 0x00, 0x00, 0x09
		$this->out->writeInt(0);
		$this->out->writeByte(9);
	}
	
	// write a RecordSet object
	function writeRecordSet ($rs)
	{	
		// create the RecordSet object
		$RecordSet = array();
		// create the serverInfo array
		$RecordSet["serverInfo"] = array();
		// create the id field --> i think this is used for pageable recordsets
		$RecordSet["serverInfo"]["id"] = "PHPRemoting";
		// get the total number of records
		$RecordSet["serverInfo"]["totalCount"] = $rs->numRows;
		// save the initial data into the RecordSet object
		$RecordSet["serverInfo"]["initialData"] = $rs->initialData;
		$RecordSet["serverInfo"]["cursor"] = 1; // maybe the current record ????
		$RecordSet["serverInfo"]["serviceName"] = "doStuff"; // in CF this is PageAbleResult not here
		$RecordSet["serverInfo"]["columnNames"] = $rs->columnNames;
		// versioning
		$RecordSet["serverInfo"]["version"] = 1;
		// write the custom class code
		$this->out->writeByte(16);
		// write the class name
		$this->out->writeUTF("RecordSet");
		// write the classes data
		$this->writeObject($RecordSet);
				
	}
	
	
	// main switch for dynamically determining the php data type
	// this may prove to be inadequate because php isn't a strictly typed
	// language and some confusion may be encountered as we discover more data types
	// to be passed back to flash
	
	// were still lacking dates, xml, and strings longer than 65536 chars
    function writeData ($d, $type=-1)
	{
	
		/***************** TO DO **********************
		Since we are now allowing the user to determine
		the datatype we have to validate the user's suggestion
		vs. the actual data being passed and throw an error
		if things don't check out.!!!!
		**********************************************/
	
		// get the type of the data if it was
		// not explicitly passed
		if ($type == -1)
		{
			$type = gettype($d);
		}
		//BOOLEANS
		if ($type == "boolean")
		{
			$this->writeBoolean($d);
		}
		//STRINGS
		else if ($type == "string")
		{
			$this->writeString($d);
		}
		// DOUBLES
		else if ($type == "double")
		{
			$this->writeNumber($d);
		}
		// INTEGERS
		else if ($type == "integer")
		{
			$this->writeNumber($d);
		}
		// OBJECTS
		else if ($type == "object")
		{
			
		}
		// ARRAYS
		else if ($type == "array")
		{
			$this->writeArray($d);
		}
		// NULL
		else if ($type == "NULL")
		{
			$this->writeNull();
		}
		// UDF's
		else if ($type == "user function")
		{
		
		}
		else if ($type == "resource")
		{
			$resource = get_resource_type($d); // determine what the resource is
			$this->writeData($d, $resource); // resend with $d's specific resource type
		}
		// XML
		else if (strtolower($type) == "xml")
		{
			$this->writeXML($d);
		}
		// Dates
		else if (strtolower($type) == "date")
		{
			$this->writeDate($d);
		}
		// mysql recordset resource
		else if (strtolower($type) == "mysql result") // resource type
		{
			// load in the mysqlRecordSet class
            include_once(AMFPHP_BASE."sql/mysqlRecordSet.php");
			// create a new recordset object
			$recordSet = new mysqlRecordSet($d); // returns formatted recordset
			// write the record set to the output stream
			$this->writeRecordSet($recordSet); // writes recordset formatted for Flash
		}		
		else
		{
			trigger_error("Unsupported Datatype");
		}
		
	}
}
?>
